unit WiNRADiO;

interface

uses
  WinTypes, WinProcs, Messages, SysUtils, Classes, Forms, ExtCtrls, Wrapi;

type
  THWInterface = (hiNone, hiISA);

  TWiNRADiO = class(TComponent)
  private
    { Private declarations }
    FActive: Boolean;
    FAtten: Boolean;
    FCWOffset: Integer;
    FFreq: Longint;
    FIFShiftLSB: Integer;
    FIFShiftUSB: Integer;
    FMode: Integer;
    FMute: Boolean;
    FPower: Boolean;
    FSignalStrength: Integer;
    FSquelch: Integer;
    FVolume: Integer;

    FOpenEvent: TNotifyEvent;
    FAttenEvent: TNotifyEvent;
    FCloseEvent: TNotifyEvent;
    FCWOffsetEvent: TNotifyEvent;
    FFreqEvent: TNotifyEvent;
    FIFShiftLSBEvent: TNotifyEvent;
    FIFShiftUSBEvent: TNotifyEvent;
    FModeEvent: TNotifyEvent;
    FMuteEvent: TNotifyEvent;
    FPowerEvent: TNotifyEvent;
    FSignalEvent: TNotifyEvent;
    FSquelchEvent: TNotifyEvent;
    FVolumeEvent: TNotifyEvent;
    FHWInterface: THWInterface;

    FModeBWList: TStrings;
    FModeList: TStrings;

    procedure SetActive(Value: Boolean);
    procedure SetAtten(Value: Boolean);
    procedure SetCWOffset(Value: Integer);
    procedure SetFreq(Value: Longint);
    procedure SetIFShiftLSB(Value: Integer);
    procedure SetIFShiftUSB(Value: Integer);
    procedure SetMode(Value: Integer);
    procedure SetMute(Value: Boolean);
    procedure SetPower(Value: Boolean);
    procedure SetSquelch(Value: Integer);
    procedure SetVolume(Value: Integer);

    function GetHasDSP: Boolean;

  protected
    { Protected declarations }
    hRadio: Integer;
    FRadioInfo: TRadioInfo;
    SquelchTimer: TTimer;
    procedure SquelchTimerEvent(Sender: TObject); virtual;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Open(DevNum: Integer): Boolean; virtual;
    procedure Close; virtual;
    function ModeToStr(AMode: Integer): string;
    function StrToMode(const AString: string): Integer;

    property DeviceNumber: Integer read FRadioInfo.iDeviceNum;
    property Handle: Integer read hRadio;
    property HasDSP: Boolean read GetHasDSP;
    property HWInterface: THWInterface read FHWInterface;
    property SignalStrength: Integer read FSignalStrength;
    property MinFreq: Longint read FRadioInfo.dwMinFreq;
    property MaxFreq: Longint read FRadioInfo.dwMaxFreq;
    property RadioInfo: TRadioInfo read FRadioInfo;
    property ModeBWs: TStrings read FModeBWList;
    property Modes: TStrings read FModeList;
  published
    { Published declarations }
    property Active: Boolean read FActive write SetActive;
    property Attenuator: Boolean read FAtten write SetAtten;
    property CWOffset: Integer read FCWOffset write SetCWOffset;
    property Frequency: Longint read FFreq write SetFreq;
    property IFShiftLSB: Integer read FIFShiftLSB write SetIFShiftLSB;
    property IFShiftUSB: Integer read FIFShiftUSB write SetIFShiftUSB;
    property Mode: Integer read FMode write SetMode;
    property Mute: Boolean read FMute write SetMute;
    property Power: Boolean read FPower write SetPower default True;
    property Squelch: Integer read FSquelch write SetSquelch;
    property Volume: Integer read FVolume write SetVolume;

    property OnOpen: TNotifyEvent read FOpenEvent write FOpenEvent;
    property OnAttenChange: TNotifyEvent read FAttenEvent write FAttenEvent;
    property OnClose: TNotifyEvent read FCloseEvent write FCloseEvent;
    property OnCWOffsetChange: TNotifyEvent read FCWOffsetEvent write FCWOffsetEvent;
    property OnFreqChange: TNotifyEvent read FFreqEvent write FFreqEvent;
    property OnIFShiftLSBChange: TNotifyEvent read FIFShiftLSBEvent write FIFShiftLSBEvent;
    property OnIFShiftUSBChange: TNotifyEvent read FIFShiftUSBEvent write FIFShiftUSBEvent;
    property OnModeChange: TNotifyEvent read FModeEvent write FModeEvent;
    property OnMuteChange: TNotifyEvent read FMuteEvent write FMuteEvent;
    property OnPowerChange: TNotifyEvent read FPowerEvent write FPowerEvent;
    property OnSignalChange: TNotifyEvent read FSignalEvent write FSignalEvent;
    property OnSquelchChange: TNotifyEvent read FSquelchEvent write FSquelchEvent;
    property OnVolumeChange: TNotifyEvent read FVolumeEvent write FVolumeEvent;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('WiNRADiO', [TWiNRADiO]);
end;

function TWiNRADiO.GetHasDSP: Boolean;
begin
  if FActive then
    Result := (RadioInfo.dwFeatures and RIF_DSP) <> 0
  else
    Result := False;
end;

procedure TWiNRADiO.SetActive(Value: Boolean);
begin
  if Value then
    Open(0)
  else
    Close;
end;

procedure TWiNRADiO.SetAtten(Value: Boolean);
begin
  if (Value <> FAtten) then
  begin
    if FActive then
    begin
      if wrapi.SetAtten(hRadio, Value) then
        FAtten := Value;
    end
    else
      FAtten := Value;
    if Assigned(FAttenEvent) then
      FAttenEvent(Self);
  end;
end;

procedure TWiNRADiO.SetCWOffset(Value: Integer);
begin
  if (Value <> FCWOffset) then
  begin
    if FActive then
    begin
      if SetBFOOffset(hRadio, Value) then
        FCWOffset := Value;
    end
    else
      FCWOffset := Value;
    if Assigned(FCWOffsetEvent) then
      FCWOffsetEvent(Self);
  end;
end;

procedure TWiNRADiO.SetFreq(Value: Longint);
begin
  if (Value <> FFreq) then
  begin
    if FActive then
    begin
      if wrapi.SetFrequency(hRadio, Value) then
        FFreq := Value;
    end
    else
      FFreq := Value;
    if Assigned(FFreqEvent) then
      FFreqEvent(Self);
  end;
end;

procedure TWiNRADiO.SetIFShiftLSB(Value: Integer);
begin
  if (Value <> FIFShiftLSB) then
  begin
    if FActive and (Lo(FMode) = RMD_LSB) then begin
      if SetIFShift(hRadio, Value) then
        FIFShiftLSB := Value;
    end
    else
      FIFShiftLSB := Value;
    if Assigned(FIFShiftLSBEvent) then
      FIFShiftLSBEvent(Self);
  end;
end;

procedure TWiNRADiO.SetIFShiftUSB(Value: Integer);
begin
  if (Value <> FIFShiftUSB) then
  begin
    if FActive and (Lo(FMode) = RMD_USB) then begin
      if SetIFShift(hRadio, Value) then
        FIFShiftUSB := Value;
    end
    else
      FIFShiftUSB := Value;
    if Assigned(FIFShiftUSBEvent) then
      FIFShiftUSBEvent(Self);
  end;
end;

procedure TWiNRADiO.SetMode(Value: Integer);
begin
  if (Value <> FMode) then
  begin
    if FActive then
    begin
      if wrapi.SetMode(hRadio, Integer(Value)) then
      begin
        FMode := Value;
        if (Lo(FMode) = RMD_LSB) then
          SetIFShift(hRadio, FIFShiftLSB)
        else
          if (Lo(FMode) = RMD_USB) then
            SetIFShift(hRadio, FIFShiftUSB);
      end;
    end
    else
      FMode := Value;
    if Assigned(FModeEvent) then
      FModeEvent(Self);
  end;
end;

procedure TWiNRADiO.SetMute(Value: Boolean);
begin
  if (Value <> FMute) then
  begin
    if FActive and (FSignalStrength >= FSquelch) then
    begin
      if wrapi.SetMute(hRadio, Value) then
        FMute := Value;
    end
    else
      FMute := Value;
    if Assigned(FMuteEvent) then
      FMuteEvent(Self);
  end;
end;

procedure TWiNRADiO.SetPower(Value: Boolean);
begin
  if (Value <> FPower) then
  begin
    if FActive then
    begin
      if wrapi.SetPower(hRadio, Value) then
        FPower := Value;
    end
    else
      FPower := Value;
    if Assigned(FPowerEvent) then
      FPowerEvent(Self);
  end;
end;

procedure TWiNRADiO.SetSquelch(Value: Integer);
begin
  if (Value <> FSquelch) then
  begin
    FSquelch := Value;
    if FActive and (not FMute) then
      wrapi.SetMute(hRadio, FSignalStrength < FSquelch);
    if Assigned(FSquelchEvent) then
      FSquelchEvent(Self);
  end;
end;

procedure TWiNRADiO.SetVolume(Value: Integer);
begin
  if (Value <> FVolume) then
  begin
    if FActive then
    begin
      if wrapi.SetVolume(hRadio, Value) then
        FVolume := Value;
    end
    else
      FVolume := Value;
    if Assigned(FVolumeEvent) then
      FVolumeEvent(Self);
  end;
end;

procedure TWiNRADiO.SquelchTimerEvent(Sender: TObject);
var
  ss: Integer;
begin
  ss := Smallint(GetSignalStrength(hRadio));
  if (ss <> FSignalStrength) then
  begin
    FSignalStrength := ss;
    if not FMute then
      wrapi.SetMute(hRadio, FSignalStrength < FSquelch);
    if Assigned(FSignalEvent) then
      FSignalEvent(Self);
  end;
end;

constructor TWiNRADiO.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FActive := False;
  FAtten := False;
  FCWOffset := 0;
  FFreq := 0;
  FIFShiftLSB := 0;
  FIFShiftUSB := 0;
  FMode := -1;
  FMute := False;
  FPower := True;
  FSignalStrength := 0;
  FSquelch := 0;
  FVolume := 0;
  FModeBWList := TStringList.Create;
  FModeList := TStringList.Create;
  with TStringList(FModeList) do
  begin
    Sorted := True;
    Duplicates := dupIgnore;
  end;
end;

function TWiNRADiO.Open(DevNum: Integer): Boolean;
var
  i: Integer;
begin
  if not FActive then
  begin
    hRadio := OpenRadioDevice(DevNum);
    if (hRadio <> 0) then
    begin
      FRadioInfo.dwSize := SizeOf(TRadioInfo);
      GetRadioDeviceInfo(hRadio, @FRadioInfo);
      if (FRadioInfo.wAPIVer < $200) then
      begin
        CloseRadioDevice(hRadio);  { need version 2.00 API at least }
        hRadio := 0;
        Open := False;
        Exit;
      end;
      if (FRadioInfo.iHWInterface >= $180) then
        FHWInterface := hiISA
      else
        FHWInterface := THWInterface(FRadioInfo.iHWInterface + 1);
      FActive := True;
      SquelchTimer := TTimer.Create(Self);
      SquelchTimer.Interval := 50;
      SquelchTimer.OnTimer := SquelchTimerEvent;
      wrapi.SetFrequency(hRadio, FFreq);
      wrapi.SetMode(hRadio, FMode);
      wrapi.SetVolume(hRadio, FVolume);
      wrapi.SetAtten(hRadio, FAtten);
      wrapi.SetMute(hRadio, FMute);
      wrapi.SetPower(hRadio, FPower);
      SetBFOOffset(hRadio, FCWOffset);
      case Lo(FMode) of
        RMD_LSB: SetIFShift(hRadio, FIFShiftLSB);
        RMD_USB: SetIFShift(hRadio, FIFShiftUSB);
      end;
      FModeBWList.Clear;
      FModeList.Clear;
      for i := 0 to FRadioInfo.iNumModes-1 do
      begin
        FModeBWList.Add(ModeToStr(FRadioInfo.lpSupportedModes^[i]));
        FModeList.Add(ModeToStr(Lo(FRadioInfo.lpSupportedModes^[i])));
      end;
      if Assigned(FOpenEvent) then
        FOpenEvent(Self);
    end;
  end;
  Open := FActive;
end;

procedure TWiNRADiO.Close;
begin
  if FActive then
  begin
    SquelchTimer.Free;
    CloseRadioDevice(hRadio);
    FActive := False;
    FModeBWList.Clear;
    FModeList.Clear;
    hRadio := 0;
    if Assigned(FCloseEvent) then
      FCloseEvent(Self);
  end;
end;

function TWiNRADiO.ModeToStr(AMode: Integer): string;
var
  s: array [0..31] of Char;
begin
  Result := StrPas(GetModeString(hRadio, AMode, s, SizeOf(s)));
end;

function TWiNRADiO.StrToMode(const AString: string): Integer;
var
  i: Integer;
begin
  Result := -1;
  for i := 0 to FModeBWList.Count-1 do
    if Pos(AString, FModeBWList[i]) = 1 then
    begin
      Result := FRadioInfo.lpSupportedModes^[i];
      Break;
    end;
end;

destructor TWiNRADiO.Destroy;
begin
  if FActive then Close;
  FModeBWList.Free;
  FModeList.Free;
  inherited Destroy;
end;

end.
